--[[
	PowerUp - Paladin Module
	
	These are the general thoughts behind how the Holy Power decay-timer works..
	
	Observation has shown that a charge expires approximately 10 seconds after you receieve it, unless you are in combat. If you are in 
	combat the charge persists (unless you use it of course) and when you exit combat a timer will start. There's one very importat 
	aspect to make note off though. If you had a charge before you entered combat, with for example 5 seconds left on its duration, and 
	you entered combat and didn't use the charge, the timer will pick up where it paused prior to combat.
	
	Due to latency both between one self and the servers as well as difference when the game considers you out of combat to when you're 
	actually out of combat (and Blizzards internal timer starts) it's not possible to make it 100% accurate. As such there's a flat .25 added
	onto the timer to account for this delay, but this number can be tweaked if wanted.
	
]]
local _, PowerUp = ...

if PowerUp.class ~= "PALADIN" then return end

local db, pguid
local powerType, powerName

-- Setup frames
function PowerUp:Init()
	db = PowerUpDB
	db.max = 3
	
	powerType = "HOLY_POWER"
	powerName = SPELL_POWER_HOLY_POWER
	
	self:Workshop()
	self:Paintshop()
	self:ToggleOverlay()
	
	self:RegisterEvent("UNIT_POWER")
	self.UNIT_POWER = self.RefreshPowers
	
	self.prior = 0
	
	-- update powers at login/zoning in case we have some already
	-- if a power disappears during loading screen, it wont update 
	-- properly otherwise.
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

local function onUpdate(self, elapsed)
	local p = self:GetProgress()
	self:GetParent().color:SetVertexColor(p, 1-p, 0, db.overlayAlpha)	
end

function PowerUp:ToggleOverlay()
	if db.showOverlay then
		if not self.timer then
			self.timer = CreateFrame("Frame", nil, self.frame)
			self.timer.color = self.timer:CreateTexture(nil, "OVERLAY")
			self.timer.color:SetTexture(db.texture)
			self.timer.color:SetAllPoints(self.timer)
			self.timer.color:SetVertexColor(db.colors.overlay.r, db.colors.overlay.g, db.colors.overlay.b, db.overlayAlpha)
			self.timer:SetAllPoints(self.frame)
			
			self.overlay = self.timer:CreateAnimationGroup()
			self.animation = self.overlay:CreateAnimation("Scale")
			self.animation:SetDuration(10.25)
			-- self.animation:SetMaxFramerate(30) -- looks nice enough
			self.animation:SetOrigin("LEFT", 0, 0)
			self.animation:SetScale(0, 1)
			
			self.overlay.Restart = function() self.overlay:Stop() self.overlay:Play() end
			self.overlay:SetScript("OnFinished", function() self.timer:Hide() end)
			self.overlay:SetScript("OnPlay", function() self.timer:Show() end)
		end
		if db.useGradient then
			self.overlay:SetScript("OnUpdate", onUpdate)
		else
			self.timer.color:SetVertexColor(db.colors.overlay.r, db.colors.overlay.g, db.colors.overlay.b, db.overlayAlpha)
			self.overlay:SetScript("OnUpdate", nil)
		end		
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")	
	else
		if self.timer then self.timer:Hide() end
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")	
	end
end

function PowerUp:PLAYER_ENTERING_WORLD()
	pguid = UnitGUID("player")
	self:RefreshPowers("player", powerType)
end

-- Timer was running, so pause it for now.
function PowerUp:PLAYER_REGEN_DISABLED()
	if self.overlay:IsPlaying() then 
		self.overlay:Pause()
	end
	self.timer:Hide()
end

-- Resume playing
function PowerUp:PLAYER_REGEN_ENABLED()
	self.overlay:Play()
end

-- Watch for changes when @ max
function PowerUp:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlagsTwo, destGUID, destName, destFlags, destFlagsTwo, _, _, _, _, power)
	if sourceGUID == pguid and event == "SPELL_ENERGIZE" and power == 9 then
		self:RefreshPowers("player", powerType)
	end
end

function PowerUp:RefreshPowers(unit, power)
	if unit ~= "player" or (power and power ~= powerType) then return end
	local num = UnitPower("player", powerName)
	
	for i = 1, db.max do
		if(i <= num) then
			self.bars[i]:SetAlpha(1)
		else
			self.bars[i]:SetAlpha(0)
		end
	end
	
	if db.showOverlay then
		if num ~= self.prior then
			-- If paused it means it was running but is now invalid, so stop it
			if self.overlay:IsPaused() then self.overlay:Stop() end
			if not InCombatLockdown() then
				if num < self.prior then -- lost a charge, check if timer should still be running
					if num > 0 then
						self.overlay:Restart()
					end
				elseif num > self.prior then -- gained a charge, restart timer
					self.overlay:Restart()
				end
			end
		end
		
		if num == db.max then
			if not InCombatLockdown() then self.overlay:Restart() end
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end			
	end
	
	if db.hidewhenempty and num == 0 then
		self.frame:Hide()
	else
		if num == 0 then self.timer:Hide() end
		self.frame:Show()
	end
	
	self.prior = num
end